unit CochranQUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Buttons, MainUnit, Globals, OutPutUnit, DataProcs, Math,
  FunctionsLib, contexthelpunit;

type

  { TCochranQFrm }

  TCochranQFrm = class(TForm)
    HelpBtn: TButton;
    ResetBtn: TButton;
    CancelBtn: TButton;
    ComputeBtn: TButton;
    ReturnBtn: TButton;
    InBtn: TBitBtn;
    Label2: TLabel;
    ListBox1: TListBox;
    OutBtn: TBitBtn;
    AllBtn: TBitBtn;
    Label1: TLabel;
    VarList: TListBox;
    procedure AllBtnClick(Sender: TObject);
    procedure ComputeBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure InBtnClick(Sender: TObject);
    procedure OutBtnClick(Sender: TObject);
    procedure ResetBtnClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  CochranQFrm: TCochranQFrm;

implementation

{ TCochranQFrm }

procedure TCochranQFrm.ResetBtnClick(Sender: TObject);
VAR i : integer;
begin
     VarList.Clear;
     ListBox1.Clear;
     for i := 1 to NoVariables do
     begin
          VarList.Items.Add(OS3MainFrm.DataGrid.Cells[i,0]);
     end;
     InBtn.Visible := true;
     OutBtn.Visible := false;
end;

procedure TCochranQFrm.FormShow(Sender: TObject);
begin
     ResetBtnClick(self);
end;

procedure TCochranQFrm.HelpBtnClick(Sender: TObject);
begin
  ContextHelpForm.HelpMessage((Sender as TButton).tag);
end;

procedure TCochranQFrm.AllBtnClick(Sender: TObject);
VAR count, index : integer;
begin
     count := VarList.Items.Count;
     if count = 0 then exit;
     for index := 0 to count-1 do
     begin
          ListBox1.Items.Add(VarList.Items.Strings[index]);
     end;
     VarList.Clear;
     InBtn.Visible := false;
     OutBtn.Visible := true;
end;

procedure TCochranQFrm.ComputeBtnClick(Sender: TObject);
var
   i, j, k, col : integer;
   ColNoSelected : IntDyneVec;
   R1, L1, L2, C1, g1, Q, g2, chiprob : double;
   cellstring, outline : string;
begin
     SetLength(ColNoSelected,NoVariables);
     C1 := 0.0;
     k := ListBox1.Items.Count;
    // Get column numbers and labels of variables selected
    for i := 1 to k do
    begin
        cellstring := ListBox1.Items.Strings[i-1];
        for j := 1 to NoVariables do
        begin
            if (cellstring = OS3MainFrm.DataGrid.Cells[j,0]) then
                ColNoSelected[i-1] := j;
        end;
    end;

    // Calculate results
    R1 := 0.0;
    L1 := 0.0;
    L2 := 0.0;
    g1 := 0.0;
    g2 := 0.0;
    for i := 1 to NoCases do
    begin
        if (not GoodRecord(i,k,ColNoSelected)) then continue;
        for j := 1 to k do
        begin
            col := ColNoSelected[j-1];
            R1 := R1 + StrToFloat(Trim(OS3MainFrm.DataGrid.Cells[col,i]));
        end;
        L1 := L1 + R1;
        L2 := L2 + (R1 * R1);
        R1 := 0.0;
    end;
    for j := 1 to k do
    begin
        for i := 1 to NoCases do
        begin
            if (not GoodRecord(i,k,ColNoSelected)) then continue;
            col := ColNoSelected[j-1];
            C1 := C1 + StrToFloat(Trim(OS3MainFrm.DataGrid.Cells[col,i]));
        end;
        g1 := g1 + C1;
        g2 := g2 + (C1 * C1);
        C1 := 0.0;
    end;

    if ((k * L1) - L2) > 0.0 then
    begin
         Q := ((k - 1) * ((k * g2) - (g1 * g1))) / ((k * L1) - L2);
         chiprob := 1.0 - chisquaredprob(Q, k - 1);
    end
    else begin
         Q := 0.0;
         chiprob := 1.0;
         ShowMessage('Error in obtaining Q and the probability.');
    end;

    //present results
    OutPutFrm.RichEdit.Clear;
    OutPutFrm.RichEdit.Lines.Add('Cochran Q Test for Related Samples');
    OutPutFrm.RichEdit.Lines.Add('See pages 161-166 in S. Siegel: Nonparametric Statistics for the Behavioral Sciences');
    OutPutFrm.RichEdit.Lines.Add('McGraw-Hill Book Company, New York, 1956');
    OutPutFrm.RichEdit.Lines.Add('');
    outline := format('Cochran Q Statistic = %6.3f',[Q]);
    OutPutFrm.RichEdit.Lines.Add(outline);
    outline := format('which is distributed as chi-square with %d D.F. and probability = %6.4f',
            [k-1, chiprob]);
    OutPutFrm.RichEdit.Lines.Add(outline);
    OutPutFrm.ShowModal;
    OutPutFrm.RichEdit.Clear;
    ColNoSelected := nil;
end;

procedure TCochranQFrm.InBtnClick(Sender: TObject);
VAR i, index : integer;
begin
     index := VarList.Items.Count;
     i := 0;
     while i < index do
     begin
         if (VarList.Selected[i]) then
         begin
            ListBox1.Items.Add(VarList.Items.Strings[i]);
            VarList.Items.Delete(i);
            index := index - 1;
            i := 0;
         end
         else i := i + 1;
     end;
     OutBtn.Visible := true;
end;

procedure TCochranQFrm.OutBtnClick(Sender: TObject);
VAR index : integer;
begin
   index := ListBox1.ItemIndex;
   VarList.Items.Add(ListBox1.Items.Strings[index]);
   ListBox1.Items.Delete(index);
   InBtn.Visible := true;
end;

initialization
  {$I cochranqunit.lrs}

end.

