unit StemLeafUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Buttons, MainUnit, Globals, OutPutUnit, DataProcs,
  DictionaryUnit, Math, Clipbrd, contexthelpunit;

type

  { TStemLeafFrm }

  TStemLeafFrm = class(TForm)
    HelpBtn: TButton;
    InBtn: TBitBtn;
    OutBtn: TBitBtn;
    AllBtn: TBitBtn;
    ResetBtn: TButton;
    CancelBtn: TButton;
    ComputeBtn: TButton;
    ReturnBtn: TButton;
    TestChk: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    VarList: TListBox;
    SelectList: TListBox;
    procedure AllBtnClick(Sender: TObject);
    procedure ComputeBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure InBtnClick(Sender: TObject);
    procedure OutBtnClick(Sender: TObject);
    procedure ResetBtnClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  StemLeafFrm: TStemLeafFrm;

implementation

{ TStemLeafFrm }

procedure TStemLeafFrm.ResetBtnClick(Sender: TObject);
var i : integer;
begin
        VarList.Clear;
        for i := 1 to NoVariables do
                VarList.Items.Add(OS3MainFrm.DataGrid.Cells[i,0]);
        SelectList.Clear;
        InBtn.Visible := true;
        OutBtn.Visible :=false;
        AllBtn.Visible := true;
end;

procedure TStemLeafFrm.FormShow(Sender: TObject);
begin
     ResetBtnClick(self);
end;

procedure TStemLeafFrm.HelpBtnClick(Sender: TObject);
begin
  ContextHelpForm.HelpMessage((Sender as TButton).tag);
end;

procedure TStemLeafFrm.AllBtnClick(Sender: TObject);
VAR i : integer;
begin
     for i := 0 to VarList.Items.Count-1 do
          SelectList.Items.Add(VarList.Items.Strings[i]);
     VarList.Clear;
     OutBtn.Visible := true;
     InBtn.Visible := false;
     AllBtn.Visible := false;
end;

procedure TStemLeafFrm.ComputeBtnClick(Sender: TObject);
var
    i, j, k, L, m, ncases, noselected, largest, smallest, howlong : integer;
    minsize, maxsize, stem, minstem, maxstem, bin, index : integer;
    leafvalue, counter, smallcount, testvalue, temp1, largestcount : integer;
    cellstring, gridstring, outline, astring : string;
    selected : IntDyneVec;
    bins : IntDyneVec;
    frequency : IntDyneVec;
    ValueString : StrDyneVec;
    values : DblDyneVec;
    leafcount : IntDyneMat;
    min, max, temp, X, stemsize : double;

begin
    noselected := SelectList.Items.Count;
    if (noselected = 0) then
    begin
        ShowMessage('ERROR! No variables were selected.');
        exit;
    end;
    SetLength(selected,noselected);
    SetLength(values,NoCases);
    SetLength(bins,100);
    SetLength(frequency,100);
    SetLength(ValueString,NoCases);
    SetLength(leafcount,100,10);

    // Get selected variables
    for i := 1 to noselected do
    begin
        cellstring := SelectList.Items.Strings[i-1];
        for j := 1 to NoVariables do
            if (cellstring = OS3MainFrm.DataGrid.Cells[j,0]) then selected[i-1] := j;
   end;

    OutPutFrm.RichEdit.Clear;
    OutPutFrm.RichEdit.Lines.Add('STEM AND LEAF PLOTS');
    OutPutFrm.RichEdit.Lines.Add('');

    // Analyze each variable selected
    for j := 0 to noselected - 1 do
    begin
         k := selected[j];
         outline := format('Stem and Leaf Plot for variable: %s',[OS3MainFrm.DataGrid.Cells[k,0]]);
         OutPutFrm.RichEdit.Lines.Add(outline);
         ncases := 0;
         min := 1.0e20;
         max := -1.0e20;
         minsize := 1000;
         maxsize := -1000;
         // Store values of the variable
         for i := 1 to NoCases do
         begin
             if not ValidValue(i,k) then continue;
             values[ncases] := StrToFloat(OS3MainFrm.DataGrid.Cells[k,i]);
             ValueString[ncases] := Trim(OS3MainFrm.DataGrid.Cells[k,i]);
             if (values[ncases] < min) then min := values[ncases];
             if (values[ncases] > max) then max := values[ncases];
             if Length(ValueString[ncases]) > maxsize then maxsize := Length(ValueString[ncases]);
             if Length(ValueString[ncases]) < minsize then minsize := Length(ValueString[ncases]);
             ncases := ncases + 1;
         end;

         largest := ceil(max);
         smallest := ceil(min);
         stemsize := 1.0;
         if ((largest > 0) and (largest > 10)) then
         begin
             while (largest > 10)do
             begin
                 largest := largest div 10;
                 stemsize := stemsize * 10.0;
             end;
         end
         else if ((largest < 0) and (smallest < -10)) then  // largest value is less than 0.0
         begin
             while (smallest < -10)do
             begin
                 smallest := smallest * 10;
                 stemsize := stemsize / 10.0;
             end;
         end;
         // rescale values by stemsize
         for i := 0 to ncases - 1 do values[i] := values[i] / stemsize;

         // multiply values by 10, round and save value divided by 10
         for i := 0 to ncases - 1 do
         begin
             temp := floor(values[i] * 10);
             temp := temp / 10.0;
             values[i] := temp;
             astring := format('%4.1f',[values[i]]);
             ValueString[i] := astring;
         end;

         // get max and min stem values for creating bins for stem values
         minstem := 999;
         maxstem := -999;
         for i := 0 to ncases - 1 do
         begin
             stem := floor(values[i]);
             if (stem < minstem) then minstem := stem;
             if (stem > maxstem) then maxstem := stem;
         end;
         // create arrays for stem and leaf plot
         for i := 0 to 19 do frequency[i] := 0;

         // sort values into ascending order
         for i := 0 to ncases-2 do
         begin
             for k := i+1 to ncases - 1 do
             begin
                 if (values[i] > values[k]) then // swap values
                 begin
                     X := values[i];
                     values[i] := values[k];
                     values[k] := X;
                     cellstring := ValueString[i];
                     ValueString[i] := ValueString[k];
                     ValueString[k] := cellstring;
                 end;
             end;
         end;
(*
         // check sizes - delete if ok
         outline := format('maxsize, minsize,stemsize: %10d %10d %10.2f',[maxsize, minsize, stemsize]);
         OutPutFrm.RichEdit.Lines.Add(outline);
         OutPutFrm.ShowModal;
         OutPutFrm.RichEdit.Clear;
*)
        if (TestChk.Checked) then
        begin   // test output
            OutPutFrm.RichEdit.Lines.Add('value     ValueString');
            for i := 0 to ncases - 1 do
            begin
                outline := format('%10.1f %s',[values[i],ValueString[i]]);
                OutPutFrm.RichEdit.Lines.Add(outline);
            end;
        end;

         OutPutFrm.RichEdit.Lines.Add('');
         OutPutFrm.RichEdit.Lines.Add('Frequency  Stem & Leaf');

         // initialize leaf count for the bins
         for i := 0 to 99 do // bins
         begin
            for k := 0 to 9 do leafcount[i,k] := 0;  // leafs 0 to 9
         end;

         // count leafs in each bin
         for i := 0 to ncases - 1 do
         begin
             bin := floor(values[i]); // truncate to get stem value
             bin := bin - minstem; // get the bin number between 0 and 100
             if (bin < 100) and (bin >= 0) then
             begin
                  bins[bin] := floor(values[i]);
                  frequency[bin] := frequency[bin] + 1; // count number of stem values
             end
             else
               begin
                   ShowMessage('Error in bin value');
                   exit;
               end;

             // get leaf value
             astring := ValueString[i];
             index := Pos('.',astring);
             leafvalue := StrToInt(astring[index+1]);
             if (leafvalue < 10) and (leafvalue >= 0) then
                leafcount[bin,leafvalue] := leafcount[bin,leafvalue] + 1
             else
               begin
                   ShowMessage('Error in leafvalue');
                   exit;
               end;
         end;

         // get max leaf counters
         largestcount := 0;
         for i := 0 to 99 do // bin
         begin
            if frequency[i] = 0 then continue; // skip empty bins
            counter := 0;
            for k := 0 to 9 do // leaf counts
            begin
               counter := counter + leafcount[i,k];
            end;
            if counter > largestcount then largestcount := counter;
         end;

         // determine leaf depth needed to get counter <= 50
         if (largestcount > 50) then
         begin
            smallcount := 2;
            testvalue := largestcount;
            while (testvalue > 50) do
            begin
                testvalue := largestcount div smallcount;
                smallcount := smallcount + 1;
            end;
            smallcount := smallcount - 1; // leaf depth needed to reduce line lengths to 50 or less
         end
         else smallcount := 1;

         // rescale leafs
         for i := 0 to 99 do // bin
         begin
             for k := 0 to 9 do // leaf
                leafcount[i,k] := leafcount[i,k] div smallcount;
         end;

         // plot results
         for i := 0 to 99 do
         begin
             if frequency[i] = 0 then continue; // skip empty bins
             begin
                 outline := format('%6d     %3d    ',[frequency[i],bins[i]]);
                 for k := 0 to 9 do
                 begin
                        if leafcount[i,k] = 0 then continue;
                        for L := 1 to leafcount[i,k] do
                        begin
                              outline := outline + format('%d',[k]);
                        end;
                 end;
                 OutPutFrm.RichEdit.Lines.Add(outline);
             end;
         end;

         // summarize values
         OutPutFrm.RichEdit.Lines.Add('');
         outline := format('Stem width := %5.2f, max. leaf depth := %2d',[stemsize,smallcount]);
         OutPutFrm.RichEdit.Lines.Add(outline);
         outline := format('Min. value := %8.3f, Max. value := %8.3f',[min,max]);
         OutPutFrm.RichEdit.Lines.Add(outline);
         outline := format('No. of good cases := %d',[ncases]);
         OutPutFrm.RichEdit.Lines.Add(outline);
         OutPutFrm.RichEdit.Lines.Add('');
    end; // next jth variable
    OutPutFrm.ShowModal;

// clean up
    frequency := nil;
    bins := nil;
    ValueString := nil;
    values := nil;
    selected := nil;
    leafcount := nil;

end;

procedure TStemLeafFrm.InBtnClick(Sender: TObject);
VAR i, index : integer;
begin
     index := VarList.Items.Count;
     i := 0;
     while i < index do
     begin
         if (VarList.Selected[i]) then
         begin
            SelectList.Items.Add(VarList.Items.Strings[i]);
            VarList.Items.Delete(i);
            index := index - 1;
            i := 0;
         end
         else i := i + 1;
     end;
     if VarList.Items.Count <= 0 then InBtn.Visible := false;
     if SelectList.Items.Count > 0 then OutBtn.Visible := true;
end;

procedure TStemLeafFrm.OutBtnClick(Sender: TObject);
VAR index : integer;
begin
        index := SelectList.ItemIndex;
        if index > 0 then
        begin
                VarList.Items.Add(SelectList.Items.Strings[index]);
                SelectList.Items.Delete(index);
        end;
        if SelectList.Items.Count = 0 then OutBtn.Visible := false;
        if VarList.Items.Count > 0 then InBtn.Visible := true;
end;

initialization
  {$I stemleafunit.lrs}

end.

